@@ -11,7 +11,6 @@ import optparse
 import tempfile
 import StringIO
 import subprocess
-#from xml.etree import ElementTree
 import xml.etree.ElementTree
 import lxml.etree
 
@@ -204,10 +203,10 @@ class LocationFactory(object):
 
 class Directive(object):
     INDENT = 3
-    def __init__(self):
-        self.domain = None
-        self.name = ''
-        self.value = ''
+    def __init__(self, domain=None, name='', value='', args=''):
+        self.domain = domain
+        self.name = name
+        self.value = value
 
         self._parent = None
         self.children = []
@@ -223,6 +222,18 @@ class Directive(object):
         self.is_update = False
         self.is_remove = False
         self.location = None
+        self.args = args
+
+    @property
+    def args(self):
+        if self.name != 'data' and self._args:
+            return self._args
+        else:
+            return ''
+
+    @args.setter
+    def args(self, msg):
+        self._args = msg
 
     @property
     def is_modify(self):
@@ -243,11 +254,13 @@ class Directive(object):
         else:
             name = self.name
 
+        try:
+            buf = '{0}.. {1}:: {2}{3}\n\n'.format(indent_space, name, self.value, self.args)
+        except UnicodeEncodeError as err:
+            print 'Failed arguments', repr(self.args)
+            buf = '{0}.. {1}:: {2}{3}\n\n'.format(indent_space, name, self.value, '')
 
-        buf = '{0}.. {1}:: {2}\n\n'.format(indent_space, name, self.value)
-        
         change = None
-        version = self.version
         if self.is_add:
             change = 'versionadded'
         elif self.is_update:
@@ -257,7 +270,7 @@ class Directive(object):
 
         if change:
             buf += '{indent}{nest}.. {change}:: {version}\n\n'.format(
-                indent=indent_space, nest=' '*self.INDENT, change=change, version=version)
+                indent=indent_space, nest=' '*self.INDENT, change=change, version='NEW')
         return buf
 
     def __eq__(self, other):
@@ -320,13 +333,6 @@ class Directive(object):
         else:
             self._bodyend = value
     
-    def calc_checksum(self):
-        with open(self.path, 'rb') as fp:
-            lines = fp.readlines()[self.bodystart:self.bodyend]
-            data = ''.join(lines)
-            hashmaker = hashlib.md5(data)
-            self.checksum = hashmaker.hexdigest()
-
 class DirectiveFactory(object):
     
     # Domain Directive Selector
@@ -364,6 +370,9 @@ class DirectiveFactory(object):
             raise ValueError('Not found name tag.')
         leaf.value = nametag.text
 
+        argsstring_tag = tree.find('argsstring')
+        leaf.args = argsstring_tag.text
+
         location = self.set_location(leaf, tree)
         
         kind = tree.get('kind')
@@ -453,23 +462,7 @@ class DirectiveStack(object):
             return self.current.indent
         except AttributeError: # if self.current is None -> top
             return 0
-        
-def calc_checksum(path):
-    def _wrap():
-        with open(path, 'rb') as fp:
-            lines = fp.readlines()
-            directive = yield
-            while True:
-                start = directive.bodystart
-                end = directive.bodyend
-                data = ''.join(lines[start:end])
-                hashobj = hashlib.md5(data)
-                directive = yield hashobj.hexdigest()
 
-    generator = _wrap()
-    generator.next() # first is dust data
-    return generator
-        
 class VirtualDirectiveParser(object):
 
     def __init__(self, path=None):
@@ -633,10 +626,9 @@ class XMLParser(VirtualDirectiveParser):
 
 class RSTParser(VirtualDirectiveParser):
     REGX_DIRECTIVE = re.compile(
-        '^\s*..\s+(?P<domain_directive>\S+)::\s+(?P<value>.*)')
-
+        '^\s*..\s+(?P<domain_directive>\S+)::\s+(?P<name>.*)$')
     REGX_INDENT = re.compile('^(?P<indent>\s*)')
-    
+
     def _parse(self):
         path = self.path
         with open(path, 'rb') as fp:
@@ -686,12 +678,21 @@ class RSTParser(VirtualDirectiveParser):
             else:
                 domain = None
                 directive = domain_directive
-            value = match.group('value')
-
+            name = match.group('name')
+            value = ''
+            args = ''
+            try:
+                index = name.index('(')
+                value = name[:index]
+                args = name[index:]
+            except (IndexError, ValueError) as err:
+                value = name
+                args = ''
             leaf = Directive()
             leaf.domain = domain
             leaf.name = directive
             leaf.value = value
+            leaf.args = args
             return leaf
         else:
             return None
@@ -1037,7 +1038,7 @@ class DirectiveEditor(Editor):
     def __init__(self, exchanger):
         self.exchanger = exchanger
 
-    def get_directives(self, fp):
+    def _get_directives(self, fp): # not use
         regx = re.compile('\.\.\s+(?P<name>\S+)\:\:\s+(?P<param>.*)')
         fp.seek(0)
         for line in fp:
@@ -1125,73 +1126,12 @@ class DirectiveEditor(Editor):
         fp.truncate()
         fp.writelines(lines)
 
-    def _edit(self, fp, **kwds): # not use
-        param_directive = dict([(param, name)
-                                for name, param in self.get_directives(fp)])
-        
-        rst = fp.name
-        xml = self.exchanger.rst2xml(rst)
-        try:
-            tree = ElementTree.parse(xml)
-        except ElementTree.ParseError as err:
-            print 'Oops!!:', xml
-            raise
-        cdef = tree.find('compounddef')
-
-        fp.read() # seek to last
-
-        if cdef is not None:
-            # class
-            for innerclass in cdef.findall('innerclass'):
-                classref = self.exchanger.absxml(innerclass.get('refid') + '.xml')
-
-                if not os.path.isfile(classref): # assert
-                    print "!? Is it no exist why? Perhaps, doxygen's bug?: {0} by {1}".format(classref, xml)
-                    break
-                
-                classtree = ElementTree.parse(classref)
-                ccdef = classtree.find('compounddef')
-                if ccdef is not None:
-                    tag = ccdef.find('compoundname')
-                    if tag is not None:
-                        name = tag.text.strip()
-                        name = name.split('::')[-1]
-                        if not name in param_directive:
-                            fp.write('\n\n.. py:class:: {0}'.format(name))
-            
-            # function and valiable
-            print '*'*300
-            for innernamespace in cdef.findall('innernamespace'):
-                nsref = self.exchanger.absxml(innernamespace.get('refid') + '.xml')
-                nstree = ElementTree.parse(nsref)
-                ccdef = nstree.find('compounddef')
-                if ccdef is not None:
-                    for section in ccdef.findall('sectiondef'):
-                        for memberdef in section.findall('memberdef'):
-
-                            kind = memberdef.get('kind')
-                            if 'variable' == kind:
-                                nametag = memberdef.find('name')
-                                if nametag is not None:
-                                    name = nametag.text
-                                    if not name in param_directive:
-                                        fp.write('\n\n.. py:data:: {0}'.format(name))
-                            elif 'function' == kind:
-                                nametag = memberdef.find('name')
-                                if nametag is not None:
-                                    name = nametag.text
-                                    if not name in param_directive:
-                                        fp.write('\n\n.. py:function:: {0}'.format(name))
-
-                            else:
-                                print 'not suport:', kind
-
 class IndexRstBuilder(object):
     def build(self, path, src_root=None):
         src_root = src_root if src_root else path
 
         editors = TreeEditor()
-            
+
         for root, dirs, files in os.walk(path):
             _index = os.path.join(root, 'index.rst')
             if not os.path.isfile(_index):
@@ -1205,7 +1145,7 @@ class Processor(object):
     def run(self, src, xml=None, rst=None):
         xml = xml if xml is not None else self.XML_DEFAULT
         rst = rst if rst is not None else self.RST_DEFAULT
-        
+
         exchanger = Exchanger(src, xml, rst)
         doxymgr = DoxyManager(exchanger.src)
         hgmgr = HgManager()
